import traceback
import requests
import logging
from gempyp.config import DefaultSettings
import logging
import sys

not_uploaded = []

def _getHeaders(bridge_token, user_name):
    """
    for getting the bridgeToken in _sendData method
    """
    return {"Content-Type": "application/json", "bridgeToken": bridge_token, "username": user_name}


def sendSuiteData(payload, bridge_token, user_name, mode="POST"):
    """
    for checking the sendSuiteData api response
    """

    response = _sendData(payload, DefaultSettings.urls["suiteExec"], bridge_token, user_name, mode)
    if response and response.status_code == 201:
        logging.info("data uploaded successfully")

def sendTestcaseData(payload, bridge_token, user_name):
    """
    for checking the sendTestCaseData api response
    """
    try:
        response = _sendData(payload, DefaultSettings.urls["testcases"], bridge_token, user_name, method="POST")
        if response and response.status_code == 201:
            logging.info("data uploaded successfully")
            if payload in not_uploaded:
                not_uploaded.remove(payload)

    ### code for rerun of unuploaded testcases
        if response.status_code != 201:
            if payload not in not_uploaded:
                not_uploaded.append(payload)

    except Exception as e:
        logging.error(traceback.format_exc())


def _sendData(payload, url, bridge_token, user_name, method="POST"):
    """
    calling the api to upload the data into database
    takes data we need to send(payload),bridgeToken,userName and method as argument
    """
    
    # Not needed anymore as we will be reuploading the data to db.
    # if DefaultSettings.count > 3:         
    #     logging.warning("Incorrect bridgetoken/username or APIs are down. Skipping Data upload.")
    #     return None

    response = requests.request(
        method=method,
        url=url,
        data=payload,
        headers=_getHeaders(bridge_token, user_name),
    )
    if response.status_code != 200 and response.status_code != 201:
        # DefaultSettings.count += 1
        logging.info("Data not uploaded...........")
    logging.info(f"status: {response.status_code}")
    return response


import os

# have to discuss about the default location
DEFAULT_GEMPYP_FOLDER = os.getcwd()
DEBUG = True
THREADS = 8
_VERSION = "1.0.0"

urls = {

    "suiteInfo": "https://apis.gemecosystem.com/suiteinfo/",
    "suiteExec": "https://apis.gemecosystem.com/suiteexe",
    "testcases": "https://apis.gemecosystem.com/testcase",

}
count = 0



import sys
from telnetlib import STATUS
import traceback
from numpy import sort
import pandas as pd



def df_compare(df_1, df_2, key,reporter):
    try:
        #print(df_1)
        #print('======================================================')
        #print(df_2)
        #print('======================================================')
        # print(df_1.eq(df_2))
        #df_1['key'] = df_1[key].agg('-'.join, axis=1)
        df_1['key'] = df_1[key].apply(lambda row: '--'.join(row.values.astype(str)), axis=1)
        df_2['key'] = df_2[key].apply(lambda row: '--'.join(row.values.astype(str)), axis=1)
        df_1.set_index('key', inplace=True)
        df_2.set_index('key', inplace=True)
        df_1.drop(key, axis=1, inplace=True)
        df_2.drop(key, axis=1, inplace=True)
        src_key_values = df_1.index.values
        tgt_key_values = df_2.index.values
        headers = list(set(list(df_1.columns)) - set(key+['key']))
        common_keys = list(set(src_key_values) & set(tgt_key_values))
        keys_only_in_src = list(set(src_key_values) - set(tgt_key_values))
        keys_only_in_tgt = list(set(tgt_key_values) - set(src_key_values))
        #print(common_keys,keys_only_in_src, keys_only_in_tgt, list(headers))
        diff_list = []
        if common_keys:
            for key_val in common_keys:
                for field in headers:
                    src_val = df_1.loc[key_val,field]
                    tgt_val = df_2.loc[key_val,field]
                    if src_val != tgt_val:
                        # reporter.addrow({'key':key_val, 'column': field, 'src_value':src_val, 'tgt_value':tgt_val,
                        # 'ROF':'Value mismatch' if type(src_val)==type(tgt_val) else 'Data type diff' })
                        reporter.addRow("keys","description is working",STATUS.PASS)
                        reporter.addMisc("keys",key_val)
                        print("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@")
                        print("Addrow is working")
        if diff_list:
            diff_df = pd.DataFrame(diff_list,
                            columns=['key','column','src_value', 'tgt_value', 'ROF'])
            diff_df.set_index('key', inplace=True)
            return {'Keys': key,'Common_keys': sorted(common_keys), 'Keys_only_in_source': sorted(keys_only_in_src),
            'Keys_only_in_target': sorted(keys_only_in_tgt),'Difference_df': diff_df.sort_index(axis=0).to_json(orient="split")}
        else:
            return -1
    except Exception:
        traceback.print_exc()


# if __name__ == '__main__':
#     array_1 = [[1,'LeBron',3,'0'],
#                         [2,'Kobe',5,'1'],
#                         [3,'Michael',6,'2'],
#                         [4,'Larr','5','3'],
#                         [5,'Magic',5,'4'],
#                         [6,'Tim',4,'5'],
#                         [7,'test1',9,'6']]
#     df_1 = pd.DataFrame(array_1, 
#                         columns=['id1','Player','Rings','id2'])
#     # Data from friend
#     array_2 = [[1,'LeBron',3,'1'],
#                         [2,'Kobe',3,'9'],
#                         [3,'Michael!',6,'2'],
#                         [4,'Larry',5,'3'],
#                         [5,'Magicl',5,'4'],
#                         [6,'Tim',4,'5'],
#                         [10,'test2',0,'6']]
#     df_2 = pd.DataFrame(array_2, 
#                         columns=['id1','Player','Rings','id2'])
#     res = df_compare(df_1, df_2, ['id1', 'id2'])
#     print(res)
#     print(res['Difference_df'])



from abc import ABC, abstractmethod
import traceback
from typing import Dict
import logging


class AbstarctBaseConfig(ABC):
    def __init__(self, *args, **kwargs):
        self._CONFIG = {}
        self.cli_config ={}
        self.total_yflag_testcase =0
        
        try:
            self.parse(*args, **kwargs)
            # filter removed from here because we need to apply filter after updating data with cli input(if given)
            # self.update()
            logging.info("----------- Xml parsing completed ------------")
        except Exception as e:
            traceback.print_exc()
            logging.error("failed to parse the config: {e}".format(e=e))

    def getSuiteConfig(self) -> Dict:
        # logging.info("^^^^^^^^^^^^^ \n {suite_data} \n^^^^^^^^^".format(suite_data=self._CONFIG["SUITE_DATA"]))
        self.filter()
        return self._CONFIG["SUITE_DATA"]

    def getTestcaseConfig(self) -> Dict:
        """reutrn the testCaseData to filter method"""
        # logging.info("--------testCaseDict--------\n {testcaseDict} \n----------".format(testcaseDict=self._CONFIG["TESTCASE_DATA"]))
        return self._CONFIG["TESTCASE_DATA"]
    
    def getSubtestcasesConfig(self) -> Dict:

        return self._CONFIG["SUBTESTCASES_DATA"]

    def getTestcaseData(self, testcaseName: str) -> Dict:
        return self._CONFIG["TESTCASE_DATA"].get(testcaseName, None)
    
    def getSubTestcaseData(self, testcaseName: str) -> Dict:
        return self._CONFIG["SUBTESTCASES_DATA"].get(testcaseName, None)

    def getTestcaseLength(self) -> int:
        """
        return the length of testCase
        """
        return len(self._CONFIG.get("TESTCASE_DATA", []))

    @abstractmethod
    def parse(self, *args, **kwargs):
        """
        override this method in xmlConfig
        """
        pass

    def filter(self):
        """
        filter the testcases that need to be ignored based on the run value and category sets
        """
        testcase_data = self.getTestcaseConfig()
        filtered_dict = {}

        ###code for passing testcases through cli jira-113
        if "TESTCASES" in self._CONFIG['SUITE_DATA']:
            test = {}
            testcase = self._CONFIG['SUITE_DATA']["TESTCASES"]
            if testcase[0] == '[':
                testcase = testcase[1:-1]
                testcase = testcase.split(",")
            else:
                testcase = testcase.split(",")
            for key, value in testcase_data.items():
                if key in testcase:
                    test[key] = value
            testcase_data = test

        for key, value in testcase_data.items():
            testcases = ""
            if(value.get("RUN_FLAG", "N").upper()=="Y" and "SUBTESTCASES" in value.keys()):
                testcases=value.get("SUBTESTCASES").split(",")
                testcases.append(key)
            if value.get("RUN_FLAG", "N").upper() != "Y":
                continue
            if value.get("RUN_FLAG", "Y").upper() == "Y":
                self.total_yflag_testcase += 1
            if self.cli_config["CATEGORY"]!=None and value.get("CATEGORY") not in self.cli_config["CATEGORY"].split(","):
                continue
            if self.cli_config["SET"]!=None and value.get("SET") not in self.cli_config["SET"].split(","):
                print(value.get("SET"))
                continue
            

            # TODO add more filters
            
            if self.cli_config["CATEGORY"]!=None and value.get("CATEGORY") not in self.cli_config["CATEGORY"].split(","):
                print(value.get("CATEGORY"))
                continue
            if self.cli_config["SET"]!=None and value.get("SET") not in self.cli_config["SET"].split(","):
                print(value.get("SET"))
                continue


            filtered_dict[key] = value
            filtered_dict_subtestcases = {}
            if(len(testcases)>0):
                for i in range(len(testcases)):
                    if(testcases[i] in testcase_data.keys()):
                        filtered_dict_subtestcases[testcases[i]]=testcase_data.get(testcases[i])
        self._CONFIG["SUBTESTCASES_DATA"] = filtered_dict_subtestcases

        self._CONFIG["TESTCASE_DATA"] = filtered_dict

    # TODO
    def update(self):
        """to update the data that is passed by cli"""
        try:
            for element in self.cli_config.keys():
                if self.cli_config[element]:
                    if str(element) in self._CONFIG['SUITE_DATA']:
                        # print(element)
                        self._CONFIG['SUITE_DATA'][element] = self.cli_config[element]
                    else:
                        self._CONFIG['SUITE_DATA'][element] = self.cli_config[element]
        except Exception as error:
            print("error occurs in update",error)
        
        



import traceback
import os
import uuid
import time
import json
import logging


def writeToReport(dvm_obj):
    """
    This function takes DVM obj,
    creates the testcase report if the runmode is debug mode,
    Creates the dictionary that is to be sent to gempyp"""
    result = {}
    if not dvm_obj.reporter.result_file_name:
        try:
            try:
                dvm_obj.reporter.finalizeReport()   ## need to test
                if dvm_obj.data.get("OUTPUT_FOLDER", dvm_obj.default_report_path) is None:
                    os.makedirs(dvm_obj.data.get("OUTPUT_FOLDER", dvm_obj.default_report_path))
            except Exception as e:
                dvm_obj.logger.info(traceback.print_exc())
            dvm_obj.reporter.json_data = dvm_obj.reporter.template_data.makeTestcaseReport()
            dvm_obj.json_data = dvm_obj.reporter.json_data
            result = dvm_obj.reporter.serialize()
        except Exception as e:
            # dvm_obj.logger.info(traceback.print_exc())
            traceback.print_exc()
    result = dvm_obj.reporter.serialize()
    output = []
    tempdict = {} 
    tc_run_id = f"{dvm_obj.tcname}_{uuid.uuid4()}"
    tempdict["tc_run_id"] = tc_run_id
    tempdict["name"] = result["NAME"]
    tempdict["category"] = dvm_obj.category
    tempdict["status"] = result["STATUS"]
    tempdict["user"] = dvm_obj.data.get("USER")
    tempdict["machine"] = dvm_obj.data.get("MACHINE")
    tempdict["product_type"] = "GEMPYP-DV"
    tempdict["steps"] = result["json_data"]['steps']
    tempdict["result_file"] = result["RESULT_FILE"]
    tempdict["start_time"] = result["START_TIME"]
    tempdict["end_time"] = result["END_TIME"]
    tempdict["ignore"] = False
    all_status = result["json_data"]["metaData"][2]
    total = 0
    for key in all_status:
        total += all_status[key]
    result["json_data"]["metaData"][2]["TOTAL"] = total

    # getting the log file ( the custom gempyp logger)
    
    tempdict["log_file"] = dvm_obj.configData.get("log_path","N.A")

    singleTestcase = {}
    singleTestcase["testcase_dict"] = tempdict
    singleTestcase["misc"] = result.get("MISC")
    singleTestcase["json_data"] = dvm_obj.json_data
    output.append(singleTestcase)
    
    return output